//
//RXIN samples serial input from SIN
//
`timescale 1 ps / 1 ps
module altr_uart_rxin #(
    parameter ACTUAL_DSIZE = 8,
	parameter EXPAND_DSIZE = 11
) (
   //Clock and Reset
   input  clk,
   input  rst_n,
   input  baud_clken,

   //CSR Interface
   input  lcr_sp,
   input  lcr_eps_l,
   input  lcr_pen_l,
   input  lcr_stop_l,
   input  [2:0] lcr_dls_l,

   //Serial Input
   input  sin_i,

   //Put Signal
   output rxin_put,

   output rxfsm_idle,

   //Shift register output
   output [EXPAND_DSIZE-1:0] rx_data

);

//bit-timer counts
logic  half_bit_time;
logic  qtr_bit_time;
logic  stop2_bit_time;
logic  [3:0] bit_timer;
logic  bit_timer_inc;

//data_cnt counts
logic  data_cnt_full;

//RXFSM state indicators
logic rxfsm_start;
logic rxfsm_data;
logic rxfsm_parity;
logic rxfsm_stop;
logic rxfsm_break1;
logic rxfsm_break2;

//break detection
logic p_rx_break;
logic rx_break;
logic rx_break_indicator;

//new load logic
logic rxfsm_load;

logic data_cnt_s_rst;
logic data_cnt_inc;
logic [3:0] data_cnt;

//shift reg output
logic [ACTUAL_DSIZE-1:0] rx_shift_reg;
logic rx_parity;
logic rx_frame_error;
logic rx_parity_error;
logic rx_stop2;

//------------------------------------------------------------------------------
//RXFSM Logic
//------------------------------------------------------------------------------
altr_uart_rxfsm rxfsm (
   .*
);

//------------------------------------------------------------------------------
//Break Detection Logic
//------------------------------------------------------------------------------

//Possible Break condition where >1 stop bits are configured
assign p_rx_break =  lcr_stop_l & rx_frame_error & ~rx_parity & (rx_shift_reg == {ACTUAL_DSIZE{1'b0}});

//Break condition where 1 stop bit is configured
assign rx_break   = ~lcr_stop_l & rx_frame_error & ~rx_parity & (rx_shift_reg == {ACTUAL_DSIZE{1'b0}});

//When both p_rx_break and rx_break are both zero - that means no break condition exists

//This will be the break indicator sent "upstream" to RXSTOR
//Break is detected at 2 points
// 1) in BREAK1 state when 1 stop bit is configured
// 2) in BREAK2 state when 1.5/2 stop bit is configured [last stop bit must be zero]
assign rx_break_indicator = (rxfsm_break1 & rx_break) | (rxfsm_break2 & ~rx_stop2);

//------------------------------------------------------------------------------
//RX_BIT_TIMER Logic
//------------------------------------------------------------------------------

//Rx Bit Timer Increment Term
assign bit_timer_inc = rxfsm_start | rxfsm_data | rxfsm_parity | rxfsm_stop | rxfsm_break1;

//Rx Bit Timer Outputs
assign half_bit_time = (bit_timer[3] == 1 && bit_timer[2] == 0 && bit_timer[1] == 0 && bit_timer[0] == 0) ? 1'b1 : 1'b0;
assign qtr_bit_time = (bit_timer[3] == 0 && bit_timer[2] == 1 && bit_timer[1] == 0 && bit_timer[0] == 0) ? 1'b1 : 1'b0;
assign stop2_bit_time = (lcr_dls_l == 3'b000) ? qtr_bit_time : half_bit_time;

//Rx Bit Timer Instantiation
altr_uart_bit_timer rx_bit_timer (
   .inc(bit_timer_inc),
   .s_rst(~bit_timer_inc),
   .*
);

//------------------------------------------------------------------------------
//RX_DATA_CNT Logic
//------------------------------------------------------------------------------

assign data_cnt_s_rst = ~rxfsm_data | data_cnt_full;
assign data_cnt_inc   = rxfsm_data & half_bit_time;

altr_uart_data_cnt rx_data_cnt (
   .s_rst(data_cnt_s_rst),
   .inc(data_cnt_inc),
   .*
);


//------------------------------------------------------------------------------
//RX OUTPUT Logic
//------------------------------------------------------------------------------

assign rx_data = {rx_parity_error & lcr_pen_l, rx_frame_error, rx_break_indicator, rx_shift_reg};

//Loading will happen in BREAK1 state under all circumstances
//except when a BREAK is detected where number of stop bits > 1
always_comb begin
   rxfsm_load = (rxfsm_break1 & ~p_rx_break) | rxfsm_break2;
end

//This logic intentionally runs on "clk" - NOT baud_clk
logic rxfsm_load_r;

always_ff @(posedge clk or negedge rst_n) begin
   if (!rst_n) rxfsm_load_r <= 0;
   else rxfsm_load_r <= rxfsm_load;
end

assign rxin_put = rxfsm_load & ~rxfsm_load_r;


//------------------------------------------------------------------------------
//RX_SHIFT_REGISTER Logic
//------------------------------------------------------------------------------
altr_uart_rx_shift  #(
   .DSIZE(ACTUAL_DSIZE)
) rx_shift (.*);

endmodule: altr_uart_rxin
